/** \file   archdep_startup_log_error.c
 * \brief   Log error message on startup
 *
 * \author  Bas Wassink <b.wassink@ziggo.nl>
 */

/*
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"
#include "archdep_defs.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lib.h"

#ifdef USE_NATIVE_GTK3K
# include "ui.h"
#endif

/* why do I even bother */
#ifdef ARCHDEP_OS_OS2
# include <os2.h>
#endif

#include "archdep_startup_log_error.h"


/** \brief  Log an error message while the log system isn't initliazed yet
 *
 * \param[in]   format  format string
 */
void archdep_startup_log_error(const char *format, ...)
{
    char *tmp;
    va_list args;

    va_start(args, format);
    tmp = lib_mvsprintf(format, args);
    va_end(args);

#ifdef USE_NATIVE_GTK3K
    ui_error(tmp);
#endif
#ifdef __OS2__
    WinMessageBox(HWND_DESKTOP, HDWND_DESKTOP, tmp,
            "SDLVICE/2 Startup Error", 0, MB_OK);
#endif

    fprintf(stderr, "%s", tmp);
    lib_free(tmp);
}
